/***
 * Excerpted from "Rails For Java Programmers"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
***/
package com.relevancellc.people.webapp.action;

import com.relevancellc.people.service.PersonManager;
import com.relevancellc.people.webapp.form.PersonForm;
import com.relevancellc.people.model.Person;
import com.relevancellc.people.Constants;
import org.apache.struts.action.*;

import javax.servlet.http.*;
import java.util.List;

/**
 * @struts.action name="personForm" path="/editPerson" scope="request"
 * validate="false" parameter="method" input="mainMenu"
 * @struts.action name="personForm" path="/savePerson" scope="request"
 * validate="true" parameter="method" input="edit"
 * @struts.action-forward name="edit" path="/WEB-INF/pages/personForm.jsp"
 * @struts.action-forward name="list" path="/WEB-INF/pages/personList.jsp"
 */
public final class PersonAction extends BaseAction {

  public ActionForward cancel(ActionMapping mapping, ActionForm form,
                              HttpServletRequest request,
                              HttpServletResponse response)
      throws Exception {
    return mapping.findForward("viewPeople");
  }

  //START:search
  public ActionForward search(ActionMapping mapping, ActionForm form,
                              HttpServletRequest request,
                              HttpServletResponse response)
      throws Exception {
    PersonManager mgr = (PersonManager) getBean("personManager"); //<label id="code.controllers.search.di"/>
    List people = mgr.getPeople(null);                            //<label id="code.controllers.search.delegate"/>
    request.setAttribute(Constants.PERSON_LIST, people);          //<label id="code.controllers.search.variable"/>
    return mapping.findForward("list");                           //<label id="code.controllers.search.implicit"/>
  }
  //END:search

  //START:log
  public ActionForward delete(ActionMapping mapping, ActionForm form,
                              HttpServletRequest request,
                              HttpServletResponse response)
      throws Exception {
    if (log.isDebugEnabled()) {
      log.debug("Entering delete method");
    }
    //END:log

    ActionMessages messages = new ActionMessages();
    PersonForm personForm = (PersonForm) form;

    // Exceptions are caught by ActionExceptionHandler
    PersonManager mgr = (PersonManager) getBean("personManager");
    mgr.removePerson(personForm.getId());

    messages.add(ActionMessages.GLOBAL_MESSAGE,
        new ActionMessage("person.deleted"));

    // save messages in session, so they'll survive the redirect
    saveMessages(request.getSession(), messages);

    return mapping.findForward("viewPeople");
  }

  //START:show_one
  public ActionForward edit(ActionMapping mapping, ActionForm form,
                            HttpServletRequest request,
                            HttpServletResponse response)
      throws Exception {
    PersonForm personForm = (PersonForm) form;
    if (personForm.getId() != null) {
      PersonManager mgr = (PersonManager) getBean("personManager");
      Person person = mgr.getPerson(personForm.getId());
      personForm = (PersonForm) convert(person);
      updateFormBean(mapping, request, personForm);
    }
    return mapping.findForward("edit");
  }
  //END:show_one

  //START:save
  public ActionForward save(ActionMapping mapping, ActionForm form,
                            HttpServletRequest request,
                            HttpServletResponse response)
      throws Exception {
    ActionMessages messages = new ActionMessages();   //<label id="code.controllers.save.messages"/>
    PersonForm personForm = (PersonForm) form;
    boolean isNew = ("".equals(personForm.getId()));
    PersonManager mgr = (PersonManager) getBean("personManager");
    Person person = (Person) convert(personForm);
    mgr.savePerson(person);
    if (isNew) {
      messages.add(ActionMessages.GLOBAL_MESSAGE,     //<label id="code.controllers.save.add.1a"/>
          new ActionMessage("person.added"));
      saveMessages(request.getSession(), messages);   //<label id="code.controllers.save.add.1b"/>
      return mapping.findForward("mainMenu");
    } else {
      messages.add(ActionMessages.GLOBAL_MESSAGE,     //<label id="code.controllers.save.add.2a"/>
          new ActionMessage("person.updated"));
      saveMessages(request, messages);                //<label id="code.controllers.save.add.2b"/>
      return mapping.findForward("viewPeople");
    }
  }
  //END:save
}
