/***
 * Excerpted from "Rails For Java Programmers"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
***/
package com.relevancellc.people.dao;

import com.relevancellc.people.model.Person;
import org.springframework.dao.DataAccessException;

import java.util.List;

/**
 * Created by Stuthulhu: Sep 1, 2006
 */
public class PersonDaoTest extends BaseDaoTestCase {
  private Person person = null;
  private PersonDao dao = null;

  public void setPersonDao(PersonDao dao) {
    this.dao = dao;
  }

  public void testGetPeople() {
    person = new Person();
    List results = dao.getPeople(person);
    assertTrue(results.size() > 0);
  }

  public void testGetPerson()
      throws Exception {
    person = new Person();
    populate(person);

    dao.savePerson(person);
    assertNotNull(person.getId());

    person = dao.getPerson(person.getId());
    assertEquals(person.getFirstName(), "Matt");
  }

  public void testSavePerson()
      throws Exception {
    person = dao.getPerson(new Long(1));
    person.setFirstName("Matt");

    person.setLastName("Last Name Updated");

    dao.savePerson(person);

    if (log.isDebugEnabled()) {
      log.debug("updated Person: " + person);
    }

    assertEquals(person.getLastName(), "Last Name Updated");
  }

  public void testAddAndRemovePerson()
      throws Exception {
    person = new Person();
    person.setFirstName("Bill");
    person.setLastName("Joy");

    dao.savePerson(person);

    assertEquals(person.getFirstName(), "Bill");
    assertNotNull(person.getId());

    if (log.isDebugEnabled()) {
      log.debug("removing person...");
    }

    dao.removePerson(person.getId());

    try {
      person = dao.getPerson(person.getId());
      fail("Person found in database");
    } catch (DataAccessException dae) {
      log.debug("Expected exception: " + dae.getMessage());
      assertNotNull(dae);
    }
  }
}