/***
 * Excerpted from "Rails For Java Programmers"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
***/
package com.relevancellc.people.service;

// START: jmock
import org.jmock.Mock;
// END: jmock
import org.springframework.orm.ObjectRetrievalFailureException;
import com.relevancellc.people.dao.PersonDao;
import com.relevancellc.people.model.Person;
import com.relevancellc.people.service.impl.PersonManagerImpl;

import java.util.*;

/**
 * Created by Stuthulhu: Sep 1, 2006
 */
public class PersonManagerTest extends BaseManagerTestCase {
  private final String personId = "1";
  private PersonManager personManager = new PersonManagerImpl();
  private Mock personDao = null;
  private Person person = null;

  //START: setup
  protected void setUp()
      throws Exception {
    super.setUp();
    personDao = new Mock(PersonDao.class);
    personManager.setPersonDao((PersonDao) personDao.proxy());
  }
  //END: setup

  protected void tearDown()
      throws Exception {
    super.tearDown();
    personManager = null;
  }

  //START: test
  public void testGetPeople()
      throws Exception {
    List results = new ArrayList();
    person = new Person();
    results.add(person);
    personDao.expects(once()).method("getPeople")   // <label id="code.unit.mock.expects"/>
        .will(returnValue(results));
    List people = personManager.getPeople(null);
    assertTrue(people.size() == 1);
    personDao.verify();                             // <label id="code.unit.mock.verify"/>
  }
  //END: test

  public void testGetPerson()
      throws Exception {
    // set expected behavior on dao
    personDao.expects(once()).method("getPerson")
        .will(returnValue(new Person()));
    person = personManager.getPerson(personId);
    assertTrue(person != null);
    personDao.verify();
  }

  public void testSavePerson()
      throws Exception {
    // set expected behavior on dao
    personDao.expects(once()).method("savePerson")
        .with(same(person)).isVoid();

    personManager.savePerson(person);
    personDao.verify();
  }

  public void testAddAndRemovePerson()
      throws Exception {
    person = new Person();

    // set required fields
    person.setFirstName("firstName");
    person.setLastName("lastName");

    // set expected behavior on dao
    personDao.expects(once()).method("savePerson")
        .with(same(person)).isVoid();
    personManager.savePerson(person);
    personDao.verify();

    // reset expectations
    personDao.reset();

    personDao.expects(once()).method("removePerson").with(eq(new Long(personId)));
    personManager.removePerson(personId);
    personDao.verify();

    // reset expectations
    personDao.reset();
    // remove
    Exception ex = new ObjectRetrievalFailureException(Person.class, person.getId());
    personDao.expects(once()).method("removePerson").isVoid();
    personDao.expects(once()).method("getPerson").will(throwException(ex));
    personManager.removePerson(personId);
    try {
      personManager.getPerson(personId);
      fail("Person with identifier '" + personId + "' found in database");
    } catch (ObjectRetrievalFailureException e) {
      assertNotNull(e.getMessage());
    }
    personDao.verify();
  }
}
