/***
 * Excerpted from "Rails For Java Programmers"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
***/
import junit.framework.TestCase;
import org.hibernate.cfg.Configuration;
import org.hibernate.*;
import org.hibernate.criterion.*;
import org.hibernate.Session;

import java.util.List;

public class AccessPatterns extends TestCase {
  static Configuration conf;
  static SessionFactory factory;

  static {
    conf = new Configuration().configure();
    factory = conf.buildSessionFactory();
  }

  public void testLookupByID() {
    //START:byID
    Quip quip = null;
    Session sess = null;
    try {
      sess = factory.openSession();
      quip = (Quip)sess.get(Quip.class, new Long(1));
    } finally {
      sess.close();
    }
    //END:byID
    assertNotNull(quip);
    assertEquals("This is quip 1", quip.getText());
  }

  public void testLoadAll() {
    Session sess = null;
    List quips = null;
    try {
      sess = factory.openSession();
      //START:all
      quips = sess.createCriteria(Quip.class).list();
      //END:all
    } finally {
      sess.close();
    }
    assertEquals(50, quips.size());
  }

  public void testRestriction() {
    Session sess = null;
    List people = null;
    try {
      sess = factory.openSession();
      //START:restriction
      people = sess.createCriteria(Person.class)
                   .add( Expression.eq("firstName", "Justin"))
                   .addOrder(Order.asc("lastName"))
                   .list();
      //END:restriction
    } finally {
      sess.close();
    }
    assertEquals(1, people.size());
  }
  
  public void testParameters() {
    Session sess = null;
    List p = null;
    String name = "Justin";
    try {
      sess = factory.openSession();
      //START:parameter
      Criterion c = Restrictions.sqlRestriction("{alias}.first_name = ?",
                                      name,
                                      Hibernate.STRING);
      p = sess.createCriteria(Person.class).add(c).list();
      //END:parameter
    } finally {
      sess.close();
    }
    assertEquals(1, p.size());
  }

  public int countPeople() {
    Session sess = null;
    try {
      sess = factory.openSession();
      Query q = sess.createQuery("select count(p) from Person p");
      return ((Integer)q.uniqueResult()).intValue();
    } finally {
      sess.close();
    }
  }
  public void testSimpleSave() {
    Session sess = null;
    int count = countPeople();
    try {
      //START:simpleSave
      Person p = new Person();
      p.setFirstName("Dave");
      p.setLastName("Thomas");
      sess = factory.openSession();
      sess.save(p);
      sess.flush();
      //END:simpleSave
    } finally {
      sess.close();
    }
    assertEquals(count + 1, countPeople());
  }

  public void testSimpleUpdate() {
    Session sess = null;
    try {
      sess = factory.openSession();
      Person p = (Person) sess.get(Person.class, new Long(2));
      sess.close();
      //START:simpleUpdate
      //p is a Person detached from a prior session
      sess = factory.openSession();
      sess.saveOrUpdate(p);
      p.setMiddleName("Brian");
      sess.flush();
      //END:simpleUpdate
    } finally {
      sess.close();
    }
  }

  public void testDelete() {
    Session sess = null;
    try {
      //START:simpleDelete
      sess = factory.openSession();
      Person p = (Person) sess.get(Person.class, new Long(2));
      sess.delete(p);
      sess.flush();
      //END:simpleDelete
    } finally {
      sess.close();
    }
  }
}
