/***
 * Excerpted from "Rails For Java Programmers"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
***/
import org.hibernate.*;
import org.hibernate.validator.*;
import junit.framework.TestCase;

import java.util.concurrent.Callable;

/**
 * Created by Stuthulhu: Aug 17, 2006
 */
public class Validations extends TestCase {
  public void assertThrows(Class exception, Callable block) {
    try {
      block.call();
      fail("Expected Exception " + exception);
    } catch (Exception e) {
      assertTrue(exception.isAssignableFrom(e.getClass()));
    }
  }

  public void testImplicitValidation() {
    final Session sess = HibernateUtil.sf.getCurrentSession();
    //START:implicit
    assertThrows(PropertyValueException.class, new Callable() {
      public Object call()
          throws Exception {
        User u = new User();
        return sess.save(u);
      }
    });
    //END:implicit
  }

  public void testExplicitValidation() {
    //START:getInvalidValues
    User u = new User();
    ClassValidator v = new ClassValidator(User.class);
    InvalidValue[] errors = v.getInvalidValues(u);
    //END:getInvalidValues
    assertEquals(2, errors.length);
  }

  //START:saveUsersInTx
  public void saveUsersInTx(User... users) {
    Session sess = HibernateUtil.sf.getCurrentSession();
    Transaction tx = sess.beginTransaction();
    try {
      for (User user: users) {
        sess.save(user);
      }
      tx.commit();
    } catch (HibernateException e) {
      tx.rollback();
      throw e;
    }
  }
  //END:saveUsersInTx

  public void testSaveSomeUsers() {
    final User u1 = new User();
    u1.setLogin("jdoe");
    u1.setEmail("contact@relevancellc.com");
    final User u2 = new User();
    assertThrows(HibernateException.class, new Callable() {
      public Object call()
          throws Exception {
        saveUsersInTx(u1, u2);
        return null;
      }
    });

  }

}
