#---
# Excerpted from "Rails For Java Programmers"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
#---
class PeopleController < ApplicationController
  
  before_filter do |controller|
    logger.debug "About to handle #{controller.action_name}"
  end
  

  
  before_filter :log_action
  def log_action 
    logger.debug "About to handle #{self.action_name}"
  end
  

  
  before_filter :log_action, :only=>[:index,:show]
  

  
  before_filter :log_action, :except=>[:index]
  

  
  caches_page :show, :edit
  
  
  cache_sweeper :person_sweeper, :only=>[:create, :update, :destroy]
  
  
  def index
    list
    render :action => 'list'
  end

  
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
  

  
  def list
    @search = params[:search]
    if @search.blank?
      @person_pages, @people = paginate :people, :per_page => 10
    else
      query = ['first_name = :search or last_name = :search', 
              {:search=>@search}]
      @person_pages, @people = paginate :people, 
                               :per_page => 10, :conditions=>query
    end
  end
  

  def show
    @person = Person.find(params[:id])
  end

  
  def edit
    @person = Person.find(params[:id])
  end

  def new
    @person = Person.new
  end
  

  
  def create
    @person = Person.new(params[:person])
    if @person.save
      flash[:notice] = 'Person was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end
  

  
  def update
    @person = Person.find(params[:id])
    if @person.update_attributes(params[:person])         
      flash[:notice] = 'Person was successfully updated.' 
      redirect_to :action => 'show', :id => @person       
    else
      render :action => 'edit'
    end
  end
  

  
  def destroy
    logger.debug('Entering delete method')
    
    Person.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
end
