#---
# Excerpted from "Rails For Java Programmers"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
#---
class QuipsController < ApplicationController
  before_filter :login_required

  
  in_place_edit_for :quip, :text
  
  
  
  READ_ACTIONS = %w(index list show)
  permit 'admin or mortal', :only=>READ_ACTIONS
  permit 'admin', :except=>READ_ACTIONS
  

  
  caches_action :show, :edit
  

  def index
    list
    render :action => 'list'
  end

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def list_with_user_filter
    quips = Quip.find(:all)
    @quips = @filter_service.filter(quips, session[:filter])
  end
  
  def list
    @quip_pages, @quips = paginate :quips, :per_page => 10
  end
  
  
  def search
    @quips = Quip.find_text_like(params[:search])
    render :partial=>'search_results' if request.xhr?
  end
  

  
  def better_search
    @quips = Quip.find_text_like(params[:search])
    render :partial=>'search_update' if request.xhr?
  end
  
  
  def recent
    @quip_pages, @quips = paginate :quips, :per_page => params[:count], :order=>'updated_at DESC'
  end
  
  def show
    @quip = Quip.find(params[:id])
  end
  
  def filtered_show
    show
  end

  def new
    @quip = Quip.new
  end
  
  def create
    
    @quip = Quip.new(params[:quip])
    
    if @quip.save
      flash[:notice] = 'Quip was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @quip = Quip.find(params[:id])
  end

  def update
    @quip = Quip.find(params[:id])
    if @quip.update_attributes(params[:quip])
      flash[:notice] = 'Quip was successfully updated.'
      redirect_to :action => 'show', :id => @quip
    else
      render :action => 'edit'
    end
  end

  def destroy
    Quip.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
end
