#---
# Excerpted from "Rails For Java Programmers"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
#---
require 'rubygems'
require_gem 'activerecord'

ActiveRecord::Base.establish_connection( 
  :adapter=>'mysql', 
  :database=>'rails4java_test'
)

class Quip < ActiveRecord::Base; end
class Person < ActiveRecord::Base; end

require 'test/unit'
class TestAccessPatterns < Test::Unit::TestCase
  def test_lookup_by_id
    
    q = Quip.find(1)
    
    assert_equal 'This is quip 1', q.text
  end
  
  def test_all
    
    quips = Quip.find(:all)
    
    assert_equal 50, quips.size
  end
  
  def test_restriction
    
    p = Person.find(:all, 
                    :conditions => "first_name = 'Justin'", 
                    :order => "last_name ASC")
    
    assert_equal [Person.find(2)], p
  end
  
  def test_parameter
    name = 'Justin'
    
    p = Person.find(:all, 
                    :conditions => ["first_name = :fname", {:fname=>name}])
    
    assert_equal [Person.find(2)], p
  end
  
  def test_method_missing
    
    p = Person.find_by_first_name('Justin')
    ps = Person.find_all_by_first_name('Justin')
    
    assert_equal Person.find(2), p
    assert_equal [Person.find(2)], ps
  end

  def test_multiple
    
    p = Person.find_by_first_name_and_last_name('Justin', 'Gehtland')
    p = Person.find_by_first_name('Justin', :order=>'last_name ASC')
    
    assert_equal Person.find(2), p
  end
  
  def test_simple_save
    
    p = Person.new
    p.first_name = 'Dave'
    p.last_name = 'Thomas'
    p.save
    
    assert p.id
  end

  def test_intialize_save
    
    p = Person.new :first_name=>'Dave', :last_name=>'Thomas'
    p.save
    
    assert p.id
  end

  def test_create
    
    p = Person.create :first_name=>'Dave', :last_name=>'Thomas'
    
    assert p.id
  end

  def test_find_or_create
    
    p = Person.find_or_create_by_first_name_and_last_name 'Dave', 'Thomas'
    
    assert p.id
  end

  def test_simple_update
    p = Person.find(2)
    
    # p is a person from somewhere
    p.middle_name = 'Brian'
    p.save!
    
  end

  def test_simple_update
    
    Person.update(2, :middle_name=>'Brian')
    
  end
  
  def test_simple_delete
    c = Person.count
    
    p = Person.find(2)
    p.destroy
    
    assert_equal c-1, Person.count
    
  end
end