#---
# Excerpted from "Rails For Java Programmers"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'
require 'quips_controller'

# Re-raise errors caught by the controller.
class QuipsController; def rescue_action(e) raise e end; end


class QuipsControllerTest < Test::Unit::TestCase
  fixtures :quips, :users, :roles, :roles_users


  include FlexMock::TestCase


  test_bad_save(Quip)

  def setup
    @controller = QuipsController.new
    @request    = ActionController::TestRequest.new
    @response   = ActionController::TestResponse.new
    login_as(:quentin)
  end

  def test_index
    get :index
    assert_response :success
    assert_template 'list'
  end

  def test_list
    get :list

    assert_response :success
    assert_template 'list'

    assert_not_nil assigns(:quips)
  end

  
  def test_list_with_user_filter
    filter = flexmock("filter")   
    filter.should_expect do |m|   
      m.filter(Array,nil).returns([quips(:quip_1)]) 
    end
    @controller.instance_variable_set('@filter_service', filter) 
    get :list_with_user_filter
    assert_equal [quips(:quip_1)], assigns(:quips)
    assert_response :success
    assert_template 'list_with_user_filter'
  end
  

  def test_list_with_user_filter_2
    filter = flexmock("filter")
    
    matcher = FlexMock.on {|args| Array === args && args.all? {|a| a.text}}
    filter.should_expect do |m|   
      m.filter(matcher,nil).returns([quips(:quip_1)]) 
    end
    
    @controller.instance_variable_set('@filter_service', filter) 
    get :list_with_user_filter
    assert_equal [quips(:quip_1)], assigns(:quips)
    assert_response :success
    assert_template 'list_with_user_filter'
  end

  def test_search
    quip11 = Quip.find_by_text('This is quip 11')
    get :search, {:search=>'11'}
    assert_response :success
    assert_template 'search'
    assert_equal [quip11], assigns(:quips)
  end
  
  def test_better_search
    quip11 = Quip.find_by_text('This is quip 11')
    get :better_search, {:search=>'11'}
    assert_response :success
    assert_template 'better_search'
    assert_equal [quip11], assigns(:quips)
  end
  
  def test_show
    get :show, :id => 1

    assert_response :success
    assert_template 'show'

    assert_not_nil assigns(:quip)
    assert assigns(:quip).valid?
  end

  def test_new
    get :new

    assert_response :success
    assert_template 'new'

    assert_not_nil assigns(:quip)
  end

  
  def test_create
    assert_difference(Quip, :count) do
      post :create, :quip => {:text=>'Test Quip'}
      assert_response :redirect
      assert_redirected_to :action => 'list'
    end
  end
  

  def test_edit
    get :edit, :id => 1

    assert_response :success
    assert_template 'edit'

    assert_not_nil assigns(:quip)
    assert assigns(:quip).valid?
  end

  def test_update
    post :update, :id => 1
    assert_response :redirect
    assert_redirected_to :action => 'show', :id => 1
  end

  def test_destroy
    assert_not_nil Quip.find(1)

    post :destroy, :id => 1
    assert_response :redirect
    assert_redirected_to :action => 'list'

    assert_raise(ActiveRecord::RecordNotFound) {
      Quip.find(1)
    }
  end
  
end
